<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\Form\Mail;

use Dedi\SyliusMailAdminPlugin\Entity\Mail;
use Dedi\SyliusMailAdminPlugin\Form\Registry\MailVariableDefinitionRegistryInterface;
use Dedi\SyliusMailAdminPlugin\Repository\MailRepositoryInterface;
use Sylius\Bundle\ResourceBundle\Form\Type\ResourceTranslationsType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\Validator\Constraints\Callback;
use Symfony\Component\Validator\Constraints\Valid;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

class MailType extends AbstractType
{
    private MailRepositoryInterface $mailRepository;

    private MailVariableDefinitionRegistryInterface $mailDefinitionRegistry;

    public function __construct(
        MailRepositoryInterface $mailRepository,
        MailVariableDefinitionRegistryInterface $mailDefinitionRegistry
    ) {
        $this->mailRepository = $mailRepository;
        $this->mailDefinitionRegistry = $mailDefinitionRegistry;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('code', TextType::class, [
                'required' => true,
                'disabled' => true,
                'label' => 'dedi_sylius_mail_admin_plugin.code',
                'constraints' => [
                    new Callback([
                        'callback' => function ($data, ExecutionContextInterface $context) {
                            $currentMailId = $context->getRoot()->getNormData()->getId();
                            if (!empty($this->mailRepository->findByCode($data, $currentMailId))) {
                                $context
                                    ->buildViolation('dedi_sylius_mail_admin_plugin.constraints.code_not_uniqe')
                                    ->addViolation();
                            }
                        },
                    ]),
                ],
            ])
            ->add('enabled', CheckboxType::class, [
                'label' => 'dedi_sylius_mail_admin_plugin.enabled',
            ])
            ->add('description', TextareaType::class, [
                'label' => 'dedi_sylius_mail_admin_plugin.description',
                'required' => false,
            ])
            ->add('translations', ResourceTranslationsType::class, [
                'entry_type' => MailTranslationType::class,
                'constraints' => [new Valid()],
            ]);
    }

    public function buildView(FormView $view, FormInterface $form, array $options): void
    {
        parent::buildView($view, $form, $options);

        /** @var Mail $mail */
        $mail = $form->getViewData();
        $view->vars['mail_variables'] = $this->mailDefinitionRegistry->getVariableDefinitionByMailCode(
            $mail->getCode()
        );
    }
}
